@@ -1,23 +1,8 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2020  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2020  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -235,7 +220,7 @@ static void event_handler_lease_obtained(struct l_dhcp6_client *client,
 	memcpy(transaction_id, client_packet + 1, sizeof(transaction_id)); \
 	memcpy(client_packet, packet, sizeof(packet)); \
 	memcpy(client_packet + 1, transaction_id, sizeof(transaction_id)); \
-	transport->rx_cb(client_packet, sizeof(packet), client) \
+	transport->rx_cb(client_packet, sizeof(packet), 0, client) \
 
 static void test_obtain_lease(const void *data)
 {
